/***********************************************************************************
FILE NAME  	: CAN.C
DESCRIPTION	: Controller Area Network 

Copyright   : 2005 Renesas Technology Europe Ltd.
Copyright   : 2005 Renesas Technology Corporation.
All Rights Reserved
***********************************************************************************/

/***********************************************************************************
Revision History
DD.MM.YYYY OSO-UID Description
06.12.2005 RTE-NPS First Release
***********************************************************************************/

/**********************************************************************************
System Includes
***********************************************************************************/

/**********************************************************************************
User Includes
***********************************************************************************/
/* sfr29.h provides common defines for widely used items. */
#include "sfr29.h"

/* CAN0.h provides prototype of CAN functions & defines */
#include "can0.h"

/**********************************************************************************
Local/private functions prototype
***********************************************************************************/
void SetupCAN0MailBox(void);
void SetCAN0BaudRate(void);
void SetCAN0Id(unsigned char Slot, unsigned long Id);
void SetCAN0AcceptanceFilters(unsigned long Can0gmr, unsigned long Can0lmar, unsigned long Can0lmbr);

/**********************************************************************************
Global variables
***********************************************************************************/

/**********************************************************************************
User Program Code
***********************************************************************************/

/**********************************************************************************
Function Name: 	InitCAN0
Description  :	CAN Initailization function.
				Setup each relevant register during reset/initialization mode.
				These inculde the CAN control register, CAN bus timing control
				register, CAN clock select register, and mask register.
				CAN messag econtro register, CAN interrupt control register and
				CAN extended ID registers are automatically cleared when the CAN
				module goes from operating mode to reset/initialization mode.
				Note that these registers cannot be set during reset/initialization mode.
				Restore the CAN module to operating mode before setting these registers.

			 The CAN registers:
			 	CAN Message Control
				CAN Interrupt
			 	CAN Id
			 can not be set during CAN Reset/ Initialisation Mode.

			 Whilst in CAN Reset/ Initialisation Mode the CAN registers:
				CAN Control
				CAN Bus Timing
				CAN Clock Select
				CAN mask
			  can be set.

Parameters   : 	none
Return value : 	none
***********************************************************************************/
void InitCAN0(void)
{
/* Enter CAN Reset/ Initialisation Mode	*/
	c0ctlr_addr.b.reset = 1;

	/* Wait until STATE_RESET = 1; */
	while(c0str_addr.b.state_reset == 0);

	/* Make sure not in Sleep Mode */
	c0ctlr_addr.b.sleep = 0;

/* Set up CAN0 I/O pins */
	/* P9.2 & P9.3 operate as CAN Tx/ Rx Pins */
	c0ctlr_addr.b.porten = 1;
	/* No Loop Back Operation */
	c0ctlr_addr.b.loopback = 0;
	/* Message Content Memory Mapping = Byte Access */
	c0ctlr_addr.b.msgorder = 1;
	/* Normal Operation */
	c0ctlr_addr.b.basiccan = 0;
	/* CAN Bus Error Interrupt Disabled */
	c0ctlr_addr.b.buserren = 0;

/* Set CAN baud rate*/
	SetCAN0BaudRate();

/* Set CAN Acceptance filters mask registers*/
	SetCAN0AcceptanceFilters(0x1fffffff, 0x1fffffff, 0x1fffffff);

/* Enter Operation Mode	*/
	c0ctlr_addr.b.reset = 0;
	/* Wait until STATE_RESET = 0; */
	while(c0str_addr.b.state_reset == 1);
	
/* Stup mailbox direction & message ID*/
	SetupCAN0MailBox();
}
/**********************************************************************************
End of function InitCAN0
***********************************************************************************/


/**********************************************************************************
Function Name:  SetupCAN0MailBox
Description  :  CAN MailBox setup function
Parameters   : 	none
Return value : 	none
***********************************************************************************/
void SetupCAN0MailBox(void)
{
/* Specify Mailboxes as Receive */
	c0mctl0.receive.recreq  = C0Slot00_STATE;	/* SLOT0 State configuration (in can0.h) */
	c0mctl1.receive.recreq  = C0Slot01_STATE;	/* SLOT1 State configuration (in can0.h) */
	c0mctl2.receive.recreq  = C0Slot02_STATE;	/* SLOT2 State configuration (in can0.h) */
	c0mctl3.receive.recreq  = C0Slot03_STATE;	/* SLOT3 State configuration (in can0.h) */
	c0mctl4.receive.recreq  = C0Slot04_STATE;	/* SLOT4 State configuration (in can0.h) */
	c0mctl5.receive.recreq  = C0Slot05_STATE;	/* SLOT5 State configuration (in can0.h) */
	c0mctl6.receive.recreq  = C0Slot06_STATE;	/* SLOT6 State configuration (in can0.h) */
	c0mctl7.receive.recreq  = C0Slot07_STATE;	/* SLOT7 State configuration (in can0.h) */
	c0mctl8.receive.recreq  = C0Slot08_STATE;	/* SLOT8 State configuration (in can0.h) */
	c0mctl9.receive.recreq  = C0Slot09_STATE;	/* SLOT9 State configuration (in can0.h) */
	c0mctl10.receive.recreq = C0Slot10_STATE;	/* SLOT10 State configuration (in can0.h)*/
	c0mctl11.receive.recreq = C0Slot11_STATE;	/* SLOT11 State configuration (in can0.h)*/
	c0mctl12.receive.recreq = C0Slot12_STATE;	/* SLOT12 State configuration (in can0.h)*/
	c0mctl13.receive.recreq = C0Slot13_STATE;	/* SLOT13 State configuration (in can0.h)*/
	c0mctl14.receive.recreq = C0Slot14_STATE;	/* SLOT14 State configuration (in can0.h)*/
	c0mctl15.receive.recreq = C0Slot15_STATE;	/* SLOT15 State configuration (in can0.h)*/

/* Set CAN Id's */
	SetCAN0Id( 0, C0Slot0Id );	/* C0Slot0ID is #defined at beginning of can0.h*/
	SetCAN0Id( 1, C0Slot1Id );	/* C0Slot1ID is #defined at beginning of can0.h*/
	SetCAN0Id( 2, C0Slot2Id );	/* C0Slot2ID is #defined at beginning of can0.h*/
	SetCAN0Id( 3, C0Slot3Id );	/* C0Slot3ID is #defined at beginning of can0.h*/
	SetCAN0Id( 5, C0Slot5Id );	/* C0Slot4ID is #defined at beginning of can0.h*/
	SetCAN0Id( 4, C0Slot4Id );	/* C0Slot5ID is #defined at beginning of can0.h*/
	SetCAN0Id( 6, C0Slot6Id );	/* C0Slot6ID is #defined at beginning of can0.h*/
	SetCAN0Id( 7, C0Slot7Id );	/* C0Slot7ID is #defined at beginning of can0.h*/
	SetCAN0Id( 8, C0Slot8Id );	/* C0Slot8ID is #defined at beginning of can0.h*/
	SetCAN0Id( 9, C0Slot9Id );	/* C0Slot9ID is #defined at beginning of can0.h*/
	SetCAN0Id(10, C0Slot10Id );	/* C0Slot10ID is #defined at beginning of can0.h*/
	SetCAN0Id(11, C0Slot11Id );	/* C0Slot11ID is #defined at beginning of can0.h*/
	SetCAN0Id(12, C0Slot12Id );	/* C0Slot12ID is #defined at beginning of can0.h*/
	SetCAN0Id(13, C0Slot13Id );	/* C0Slot13ID is #defined at beginning of can0.h*/
	SetCAN0Id(14, C0Slot14Id );	/* C0Slot14ID is #defined at beginning of can0.h*/
	SetCAN0Id(15, C0Slot15Id );	/* C0Slot15ID is #defined at beginning of can0.h*/
}
/**********************************************************************************
End of function SetupCAN0MailBox
***********************************************************************************/

/**********************************************************************************
Function Name:  set_bitrate_can0
Description  :  CAN baud rate setup function
Parameters   : 	none
Return value : 	none
***********************************************************************************/
void SetCAN0BaudRate(void)
{
	/* Refer to 'can.h' for Baud Rate #defines that specify CAN Baud Rate */
	union {
		unsigned char byte;
		struct {
			unsigned char b0:1;
			unsigned char b1:1;
			unsigned char b2:1;
			unsigned char b3:1;
			unsigned char b4:1;
			unsigned char b5:1;
			unsigned char b6:1;
			unsigned char b7:1;
		} bit;
	} convert;

	/* Access to CM0, CM1, CM2, PLC0, PCLKR, CCLKR registers */
	prcr_addr.bit.b0 = 1;	/*	prc0              */
							/* 	0 - Disable write */
							/* 	1 - Enable write  */

	/* CAN0 CLOCK DIVIDER */
	convert.byte = CAN0_CLOCK_DIVIDER;

	cclkr_addr.bit.b2 = convert.bit.b2;		/*	b2	b1	b0 */
	cclkr_addr.bit.b1 = convert.bit.b1;		/* 	0	0	0	: No Division */
	cclkr_addr.bit.b0 = convert.bit.b0;		/* 	0	0	1	: fCAN = System Clk Divide by 2 */
											/* 	0	1	0	: fCAN = System Clk Divide by 4 */
											/* 	0	1	1	: fCAN = System Clk Divide by 8 */
											/* 	1	0	0	: fCAN = System Clk Divide by 16*/
											/* 	1	0	1	: DO NOT SET */
											/* 	1	1	0	: DO NOT SET */
											/* 	1	1	1	: DO NOT SET */

	/*Access to PM0, PM1, PM2, TB2SC, INVC0, INVC1 registers */
	prcr_addr.bit.b0 = 0;	/*	prc0              */
							/* 	0 - Disable write */
							/* 	1 - Enable write  */

	/* BAUD RATE prescaler */
	c0conr_addr.b.brp = CAN0_BAUD_RATE_PRESCALER;	/* 0: fCANCLK = fCAN Divide by 1 */
													/* 1: fCANCLK = fCAN Divide by 2 */
													/* 2: fCANCLK = fCAN Divide by 3 */
													/* . */
													/* . */
													/* . */
													/* 14: fCANCLK = fCAN Divide by 15 */
													/* 15: fCANCLK = fCAN Divide by 16 */

	c0conr_addr.b.sam = CAN0_SAMPLE_POINT;			/* 0: 1 Sample Points */
													/* 1: 3 Sample Points */

	c0conr_addr.b.pts = CAN0_PROP_TIME_SEG;		    /* Propagation Time Segment */
													/* 0 = 1 Tq */
													/* 1 = 2 Tq */
													/* 2 = 3 Tq */
													/* 3 = 4 Tq */
													/* 4 = 5 Tq */
													/* 5 = 6 Tq */
													/* 6 = 7 Tq */
													/* 7 = 8 Tq */

	c0conr_addr.b.pbs1 = CAN0_PHASE_BUFF_SEG1;		/* Phase Buffer Segment 1 */
							 						/* 0 = Do not Set Value */
													/* 1 = 2 Tq*/
													/* 2 = 3 Tq*/
													/* 3 = 4 Tq*/
													/* 4 = 5 Tq*/
													/* 5 = 6 Tq*/
													/* 6 = 7 Tq*/
													/* 7 = 8 Tq*/

	c0conr_addr.b.pbs2 = CAN0_PHASE_BUFF_SEG2;		/* Phase Buffer Segment 2*/
							 						/* 0 = Do not Set Value*/
													/* 1 = 2 Tq*/
													/* 2 = 3 Tq*/
													/* 3 = 4 Tq*/
													/* 4 = 5 Tq*/
													/* 5 = 6 Tq*/
													/* 6 = 7 Tq*/
													/* 7 = 8 Tq*/

	c0conr_addr.b.sjw = CAN0_RE_SYNC_JUNP_WIDTH;	/* Resynchronization Jump Width*/
													/* 0: 1 Tq*/
													/* 1: 2 Tq*/
													/* 2: 3 Tq*/
													/* 3: 4 Tq*/

}
/**********************************************************************************
End of function SetCAN0BaudRate
***********************************************************************************/

/**********************************************************************************
Function Name:  SetCAN0Id
Description  :  Setup slot / message box
Parameters   : 	Slot : Slot / message box
				ID   : Message Identification.
Return value : 	none
***********************************************************************************/
void SetCAN0Id(unsigned char Slot, unsigned long Id)
{
	union {
		unsigned char uc[4];
		unsigned short us[2];
		unsigned long ul;
	} Convert;

	/* Provide access to ID components */
	Convert.ul = Id;

		if(c0ctlr_addr.b.msgorder == 1)
	{
		switch(Slot)
		{
			case 0:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot0 = 1;
					c0slot0.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot0.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6]*/
					Convert.ul = (Convert.ul << 2);
					c0slot0.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot0.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14]*/
					Convert.ul = (Convert.ul << 4);
					c0slot0.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot0 = 0;
					c0slot0.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot0.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 1:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot1 = 1;
					c0slot1.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0]*/
					c0slot1.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6]*/
					Convert.ul = (Convert.ul << 2);
					c0slot1.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot1.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14]*/
					Convert.ul = (Convert.ul << 4);
					c0slot1.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot1 = 0;
					c0slot1.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0]*/
					Convert.ul = (Convert.ul << 2);
					c0slot1.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6]*/
				}
				break;
			}

			case 2:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame*/
					c0idr_addr.b.slot2 = 1;
					c0slot2.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0]*/
					c0slot2.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6]*/
					Convert.ul = (Convert.ul << 2);
					c0slot2.ba.eidm = Convert.uc[1];			/* EID[13:6]*/
					c0slot2.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14]*/
					Convert.ul = (Convert.ul << 4);
					c0slot2.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0]*/
				}
				else
				{
					/* Standard Frame*/
					c0idr_addr.b.slot2 = 0;
					c0slot2.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0]*/
					Convert.ul = (Convert.ul << 2);
					c0slot2.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6]*/
				}
				break;
			}

			case 3:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame*/
					c0idr_addr.b.slot3 = 1;
					c0slot3.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0]*/
					c0slot3.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6]*/
					Convert.ul = (Convert.ul << 2);
					c0slot3.ba.eidm = Convert.uc[1];			/* EID[13:6]*/
					c0slot3.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14]*/
					Convert.ul = (Convert.ul << 4);
					c0slot3.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0]*/
				}
				else
				{
					/* Standard Frame*/
					c0idr_addr.b.slot3 = 0;
					c0slot3.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0]*/
					Convert.ul = (Convert.ul << 2);
					c0slot3.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 4:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame  */
					c0idr_addr.b.slot4 = 1;
					c0slot4.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot4.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot4.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot4.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot4.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot4 = 0;
					c0slot4.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot4.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 5:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot5 = 1;
					c0slot5.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot5.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot5.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot5.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot5.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot5 = 0;
					c0slot5.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot5.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 6:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot6 = 1;
					c0slot6.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot6.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot6.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot6.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot6.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot6 = 0;
					c0slot6.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot6.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 7:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot7 = 1;
					c0slot7.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot7.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot7.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot7.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot7.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot7 = 0;
					c0slot7.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot7.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 8:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot8 = 1;
					c0slot8.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot8.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot8.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot8.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot8.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot8 = 0;
					c0slot8.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot8.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 9:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot9 = 1;
					c0slot9.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot9.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot9.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot9.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot9.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot9 = 0;
					c0slot9.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot9.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 10:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot10 = 1;
					c0slot10.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot10.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot10.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot10.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot10.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot10 = 0;
					c0slot10.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot10.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 11:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot11 = 1;
					c0slot11.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot11.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot11.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot11.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot11.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot11 = 0;
					c0slot11.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot11.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 12:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot12 = 1;
					c0slot12.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot12.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot12.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot12.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot12.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot12 = 0;
					c0slot12.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot12.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 13:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot13 = 1;
					c0slot13.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot13.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot13.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot13.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot13.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot13 = 0;
					c0slot13.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot13.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 14:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot14 = 1;
					c0slot14.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot14.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot14.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot14.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot14.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot14 = 0;
					c0slot14.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot14.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			case 15:
			{
				if(Convert.ul > 0x000007ff)
				{
					/* Extended Frame */
					c0idr_addr.b.slot15 = 1;
					c0slot15.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
					c0slot15.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
					Convert.ul = (Convert.ul << 2);
					c0slot15.ba.eidm = Convert.uc[1];			/* EID[13:6] */
					c0slot15.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
					Convert.ul = (Convert.ul << 4);
					c0slot15.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
				}
				else
				{
					/* Standard Frame */
					c0idr_addr.b.slot15 = 0;
					c0slot15.ba.sidl = (Convert.uc[0] & 0x3f);	/* SID[5:0] */
					Convert.ul = (Convert.ul << 2);
					c0slot15.ba.sidh = (Convert.uc[1] & 0x1f);	/* SID[10:6] */
				}
				break;
			}

			default:
			{
				break;
			}
		}
	}
}
/**********************************************************************************
End of function SetCAN0Id
***********************************************************************************/


/**********************************************************************************
Function Name:  SetCAN0AcceptanceFilters
Description  :  Setup acceptance filters mask values
Parameters   : 	Can0gmr  : C0GMR registr value
				Can0lmar : C0MAR registr value
				Can0lmbr : C0MBR registr value

Return value : 	none
***********************************************************************************/
void SetCAN0AcceptanceFilters(unsigned long Can0gmr, unsigned long Can0lmar, unsigned long Can0lmbr)
{
	union
	{
		unsigned char uc[4];		/*	< uc[3] >< uc[2] >< uc[1] >< uc[0] > */
		unsigned short us[2];		/*	<      us[1]     ><      us[0]     > */
		unsigned long ul;			/*	<                ul                > */
	} Convert;

	if(c0ctlr_addr.b.msgorder == 1)
	{
		Convert.ul = Can0gmr;
		c0gmr.ba.eidl = (Convert.uc[0] & 0x3f);		/* EID[5:0] */
		c0gmr.ba.sidh = (Convert.uc[3] & 0x1f);		/* SID[10:6] */
		Convert.ul = (Convert.ul << 2);
		c0gmr.ba.eidm = Convert.uc[1];				/* EID[13:6] */
		c0gmr.ba.eidh = (Convert.uc[2] & 0x0f);		/* EID[17:14] */
		Convert.ul = (Convert.ul << 4);
		c0gmr.ba.sidl = (Convert.uc[3] & 0x3f);		/* SID[5:0] */

		Convert.ul = Can0lmar;
		c0lmar.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
		c0lmar.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
		Convert.ul = (Convert.ul << 2);
		c0lmar.ba.eidm = Convert.uc[1];				/* EID[13:6] */
		c0lmar.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
		Convert.ul = (Convert.ul << 4);
		c0lmar.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */

		Convert.ul = Can0lmbr;
		c0lmbr.ba.eidl = (Convert.uc[0] & 0x3f);	/* EID[5:0] */
		c0lmbr.ba.sidh = (Convert.uc[3] & 0x1f);	/* SID[10:6] */
		Convert.ul = (Convert.ul << 2);
		c0lmbr.ba.eidm = Convert.uc[1];				/* EID[13:6] */
		c0lmbr.ba.eidh = (Convert.uc[2] & 0x0f);	/* EID[17:14] */
		Convert.ul = (Convert.ul << 4);
		c0lmbr.ba.sidl = (Convert.uc[3] & 0x3f);	/* SID[5:0] */
	}
}
/**********************************************************************************
End of function SetCAN0AcceptanceFilters
***********************************************************************************/


/**********************************************************************************
Function Name:  WriteCAN0Message
Description  :  Write message to CAN.
Parameters   : 	Slot   : Slot / Message box No.(Valid 0 to 15 only)
				Length : Message length (Valid 1 to 8 only)
				ptrData: Pointer to Write Data buffer

Return value : 	Error Status
 				NO_ERROR : on successful write.
				LENGTH_ERROR: if error lenght is invalid (Valid 1 to 8 only)
				SLOT_ERROR: if slot is not correct (Valid 0 to 15 only)
***********************************************************************************/
int WriteCAN0Message(unsigned char Slot, unsigned char Length, unsigned char *ptrData)
{
	int iStatus = NO_ERROR;
	unsigned char Count;

	if( Length > 8 || Length <= 0 )
	{
		/* Length Error*/
		iStatus = LENGTH_ERROR;
	}
	else
	{
		switch(Slot)
		{
			case 0:
			{
				if(Length > 0)
				{
					c0slot0.ba.dlc = Length;
					c0mctl0.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot0.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 1:
			{
				if(Length > 0)
				{
					c0slot1.ba.dlc = Length;
					c0mctl1.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot1.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 2:
			{
				if(Length > 0)
				{
					c0slot2.ba.dlc = Length;
					c0mctl2.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot2.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 3:
			{
				if(Length > 0)
				{
					c0slot3.ba.dlc = Length;
					c0mctl3.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot3.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 4:
			{
				if(Length > 0)
				{
					c0slot4.ba.dlc = Length;
					c0mctl4.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot4.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 5:
			{
				if(Length > 0) {
					c0slot5.ba.dlc = Length;
					c0mctl5.transmit.remote = 0;
					for(Count=0; Count < Length; Count++) {
						c0slot5.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 6:
			{
				if(Length > 0)
				{
					c0slot6.ba.dlc = Length;
					c0mctl6.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot6.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 7:
			{
				if(Length > 0)
				{
					c0slot7.ba.dlc = Length;
					c0mctl7.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot7.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 8:
			{
				if(Length > 0)
				{
					c0slot8.ba.dlc = Length;
					c0mctl8.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot8.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 9:
			{
				if(Length > 0)
				{
					c0slot9.ba.dlc = Length;
					c0mctl9.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot9.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 10:
			{
				if(Length > 0)
				{
					c0slot10.ba.dlc = Length;
					c0mctl10.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot10.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}

			case 11:
			{
				if(Length > 0)
				{
					c0slot11.ba.dlc = Length;
					c0mctl11.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot11.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}
			case 12:
			{
				if(Length > 0)
				{
					c0slot12.ba.dlc = Length;
					c0mctl12.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot12.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}
			case 13:
			{
				if(Length > 0)
				{
					c0slot13.ba.dlc = Length;
					c0mctl13.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot13.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}
			case 14:
			{
				if(Length > 0)
				{
					c0slot14.ba.dlc = Length;
					c0mctl14.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot14.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}
			case 15:
			{
				if(Length > 0)
				{
					c0slot15.ba.dlc = Length;
					c0mctl15.transmit.remote = 0;
					for(Count=0; Count < Length; Count++)
					{
						c0slot15.ba.data[Count] = *ptrData++;
					}
				}
				break;
			}
			default:
			{			
				/* Slot Error*/
				iStatus = SLOT_ERROR;
			}
			break;
		} /* End of switch*/
	} /* End of if*/

 return(iStatus);
}
/**********************************************************************************
End of function WriteCAN0Message
***********************************************************************************/


/**********************************************************************************
Function Name:  SendCAN0Message
Description  :  Starts actual message transmission on CAN bus.
Parameters   : 	Slot   : Slot / Message box No.(Valid 0 to 15 only)

Return value : 	Error Status
 				NO_ERROR : on successful write.
				SLOT_ERROR: if slot is not correct (Valid 0 to 15 only)
***********************************************************************************/
int SendCAN0Message(unsigned char Slot)
{
	int iStatus = NO_ERROR;
	unsigned long ulSendDelay = SENDING_DELAY;

	/* When starting next ransmission, set the SentData and TrmReq bits to "0" 
	   And set TrmReq bit to "1" after checking that the SentData and TrmReq bits are set to "0".
	 */
	switch(Slot)
	{
		case  0: 				
		{
			 c0mctl0.transmit.trmreq   = 0; 	
			 c0mctl0.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl0.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl0.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case  1:  				
		{
			 c0mctl1.transmit.trmreq   = 0; 	
			 c0mctl1.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl1.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
				c0mctl1.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case  2:  				
		{
			 c0mctl2.transmit.trmreq   = 0; 	
			 c0mctl2.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl2.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl2.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case  3:  				
		{
			 c0mctl3.transmit.trmreq   = 0; 	
			 c0mctl3.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl3.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl3.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case  4:  				
		{
			 c0mctl4.transmit.trmreq   = 0; 	
			 c0mctl4.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl4.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl4.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case  5:  				
		{
			 c0mctl5.transmit.trmreq   = 0; 	
			 c0mctl5.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl5.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl5.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case  6:  				
		{
			 c0mctl6.transmit.trmreq   = 0; 	
			 c0mctl6.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl6.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl6.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case  7:  				
		{
			 c0mctl7.transmit.trmreq   = 0; 	
			 c0mctl7.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl7.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl7.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case  8:  				
		{
			 c0mctl8.transmit.trmreq   = 0; 	
			 c0mctl8.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl8.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl8.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case  9:  				
		{
			 c0mctl9.transmit.trmreq   = 0; 	
			 c0mctl9.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl9.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl9.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case 10:  				
		{
			 c0mctl10.transmit.trmreq   = 0; 	
			 c0mctl10.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl10.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl10.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case 11:  				
		{
			 c0mctl11.transmit.trmreq   = 0; 	
			 c0mctl11.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl11.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl11.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case 12:  				
		{
			 c0mctl12.transmit.trmreq   = 0; 	
			 c0mctl12.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl12.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl12.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case 13:  				
		{
			 c0mctl13.transmit.trmreq   = 0; 	
			 c0mctl13.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl13.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl13.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case 14:  				
		{
			 c0mctl14.transmit.trmreq   = 0; 	
			 c0mctl14.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl14.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl14.transmit.trmreq = 1; 	
			 }
			 break;
		}
		case 15:  				
		{
			 c0mctl15.transmit.trmreq   = 0; 	
			 c0mctl15.transmit.sentdata = 0; 
			 /*Wait for transmission slot to become empty */
			 while( (c0mctl15.transmit.trmactive == 1) && (--ulSendDelay));	
			 if(ulSendDelay == 0)
			 {
			 	iStatus = TIMEOUT_ERROR;
			 }
			 else
			 {
			 	c0mctl15.transmit.trmreq = 1; 	
			 }
			 break;
		}
		default:
		{
			 iStatus = SLOT_ERROR;
			 break;
		}
	}

 return(iStatus);
} 
/**********************************************************************************
End of function SendCAN0Message
***********************************************************************************/

/**********************************************************************************
Function Name:  ReadCAN0Message
Description  :  Strats actual message transmission on CAN bus.
Parameters   : 	Slot       : Slot / Message box No.(Valid 0 to 15 only)
				pRxDataBuff: Receive data buffer( pre allocated memory)

Return value : 	Error Status
 				NO_ERROR : on successful write.
				SLOT_ERROR: if slot is not correct (Valid 0 to 15 only)
***********************************************************************************/
int ReadCAN0Message(unsigned char Slot, unsigned char* pRxDataBuff)
{
	/* Copy received data from message slots to memory */ 

	unsigned char Count;
	int iStatus = NO_ERROR;

	switch(Slot)
	{
		case 0:
			for(Count=0; Count<c0slot0.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot0.ba.data[Count];
			}
		break;

		case 1:
			for(Count=0; Count<c0slot1.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot1.ba.data[Count];
			}
		break;

		case 2:
			for(Count=0; Count<c0slot2.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot2.ba.data[Count];
			}
		break;

		case 3:
			for(Count=0; Count<c0slot3.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot3.ba.data[Count];
			}
		break;

		case 4:
			for(Count=0; Count<c0slot4.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot4.ba.data[Count];
			}
		break;

		case 5:
			for(Count=0; Count<c0slot5.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot5.ba.data[Count];
			}
		break;

		case 6:
			for(Count=0; Count<c0slot6.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot6.ba.data[Count];
			}
		break;

		case 7:
			for(Count=0; Count<c0slot7.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot7.ba.data[Count];
			}
		break;

		case 8:
			for(Count=0; Count<c0slot8.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot8.ba.data[Count];
			}
		break;

		case 9:
			for(Count=0; Count<c0slot9.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot9.ba.data[Count];
			}
		break;

		case 10:
			for(Count=0; Count<c0slot10.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot10.ba.data[Count];
			}
		break;

		case 11:
			for(Count=0; Count<c0slot11.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot11.ba.data[Count];
			}
		break;

		case 12:
			for(Count=0; Count<c0slot12.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot12.ba.data[Count];
			}
		break;

		case 13:
			for(Count=0; Count<c0slot13.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot13.ba.data[Count];
			}
		break;

		case 14:
			for(Count=0; Count<c0slot14.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot14.ba.data[Count];
			}
		break;

		case 15:
			for(Count=0; Count<c0slot15.ba.dlc; Count++)
			{
				pRxDataBuff[Count] = c0slot15.ba.data[Count];
			}
		break;

		default:
			iStatus = SLOT_ERROR;
		break;
	}
 return(iStatus);
}
/**********************************************************************************
End of function ReadCAN0Message
***********************************************************************************/

/**********************************************************************************
Function Name:  PollCAN0Message
Description  :  Polls actual message on CAN bus.
Parameters   : 	Slot       : Slot / Message box No.(Valid 0 to 15 only)

Return value : 	Error Status
 				NO_ERROR : on successful write.
				SLOT_ERROR: if slot is not correct (Valid 0 to 15 only)
				TIMEOUT_ERROR: if time out
***********************************************************************************/
int PollCAN0Message(unsigned char Slot)
{
	int iStatus = NO_ERROR;
	unsigned long ulPollDelay = POLLING_DELAY;

	switch(Slot)
	{
		case 0:
				while((c0mctl0.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 1:
				while((c0mctl1.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
		case 2:
				while((c0mctl2.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 3:
				while((c0mctl3.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 4:
				while((c0mctl4.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 5:
				while((c0mctl5.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 6:
				while((c0mctl6.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 7:
				while((c0mctl7.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 8:
				while((c0mctl8.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 9:
				while((c0mctl9.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 10:
				while((c0mctl10.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 11:
				while((c0mctl11.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 12:
				while((c0mctl12.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 13:
				while((c0mctl13.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 14:
				while((c0mctl14.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		case 15:
				while((c0mctl15.receive.newdata == 0) && (--ulPollDelay));
				if(ulPollDelay == 0) iStatus = TIMEOUT_ERROR;
				break;
		default:
				iStatus = SLOT_ERROR;
			    break;
	}

	return(iStatus);
}
/**********************************************************************************
End of function PollCAN0Message
***********************************************************************************/


/**********************************************************************************
Function Name:  Check_err_can01
Description  :  Checks CAN error status.
Parameters   : 	None
Return value : 	Error Status
 				0 : No error occur
				1 : can0 bus error
				2 : can0 is in error passive state
				4 : can0 is in bus-off state
***********************************************************************************/
int Check_err_can01(void)
{
	int iStatus = NO_ERROR; /* Store return value */

	/* Check CAN0 error state */
	if(c0str_addr.b.state_buserror)
	{
		iStatus |= 0x0001;

		/* Check error-passive state */
		if(c0str_addr.b.state_errpas)
		{
			iStatus |= 0x0002;
		}

		/* Check bus-off state */
		else if(c0str_addr.b.state_busoff)
		{
			iStatus |= 0x0004;
		}
	}

	return(iStatus);
}
/**********************************************************************************
End of function Check_err_can01
***********************************************************************************/
